/***** code.h *************************************
 * Description: 
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Tue Nov  1 18:24:52 2016
 **************************************************/
#ifndef CODE
#define CODE
#include "gsl_rng.h" 
/* define argument container */
typedef struct code{
  int  ***tri; /* codon triplets */
  int    *map; /* mapping codon->aa */
  char  **aa3; /* amino acids as 3-char strings */
  char  **aa1; /* amino acids as 1-char strings */
  char   *nuc; /* nucleotides */
  double *pro; /* amino acid properties */
  double  ms0; /* mean squared change over all codon positions */
  double  ms1; /* mean squared change at first codon position */
  double  ms2; /* mean squared change at second codon position */
  double  ms3; /* mean squared change at third codon position */
  int     fn1; /* normalization factor first codon position */
  int     fn2; /* normalization factor second condon position */
  int     fn3; /* normalization factor third codon position */
  int      n1; /* number of aa pairs for first position */
  int      n2; /* number of aa pairs for second position */
  int      n3; /* number of aa pairs for third position */
  int **pairs1;/* pairs of non-synonymous mutations at third codon position */
  int **pairs2;/* pairs of non-synonymous mutations at second codon position */
  int **pairs3;/* pairs of non-synonymous mutations at third codon position */
} Code;

void freeCode(Code *c);
Code *getCode(FILE *fp);
void printCode(Code *c);
char *codonToAa1(int codon, Code *code);
char *codonToAa3(int codon, Code *code);
double codonToProp(int codon, Code *code);
void shuffleMap(Code *c, gsl_rng *ran);
void analyzeCode(Code *c);
#endif
