/***** interface.c ************************************************
 * Description: Routine for gathering arguments from the command
 *              line.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Sun Jun 20 13:12:10 2004.
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "interface.h"
#include "eprintf.h"

Args *args;

Args *getArgs(int argc, char *argv[]){
  int c;
  char *optString = "hvdqpmn:s:";

  args = (Args *)emalloc(sizeof(Args));
  args->h = 0;
  args->v = 0;
  args->e = 0;
  args->d = 0;
  args->n = DEFAULT_N;
  args->s = 0;
  args->q = 0;
  args->p = 0;
  args->m = 0;

  c = getopt(argc, argv, optString);
  while(c != -1){
    switch(c){
    case 'd':                           /* debugging mode */
      args->d = 1;
      break;
    case 'n':                           /* number of iterations */
      args->n = atoi(optarg);
      break;
    case 's':
      args->s = atoi(optarg);
      break;
    case 'q':
      args->q = 1;
      break;
    case 'm':                           /* print m1, m2, and m3 in addition to ms0 at the end */
      args->m = 1;
      break;
    case 'p':                           /* print all ms0 vales */
      args->p = 1;
      break;
    case '?':                           /* fall-through is intentional */
    case 'h':                           /* print help */
      args->h = 1;
      break;
    case 'v':                           /* print version */
      args->v = 1;
      break;
    default:
      printf("# unknown argument: %c\n",c);
      args->e = 1;
      return args;
    }
    c = getopt(argc, argv, optString);
  }
  args->inputFiles = argv + optind;
  args->numInputFiles = argc - optind;
  return args;
}


void printUsage(char *version){
  printf("Usage: %s [options] [inputFiles]\n",progname());
  printf("Generate random genetic codes and measure their conservation\n");
  printf("Reference: Haig & Hurst (1991). A quantitative measure of error\n");
  printf("\tminimization in the genetic code. J. Mol. Evol. 33:412-417\n");
  printf("Example: genCode polarity.dat\n");
  printf("Options:\n");
  printf("\t[-n <NUM> number of iterations; default: %d]\n",DEFAULT_N);
  printf("\t[-s <NUM> seed for random number generator; default: system clock, randomSeed.dat]\n");
  printf("\t[-p print all ms0 values; default: only ms0 of better codes]\n");
  printf("\t[-q quiet, do not print better codes; default: print them]\n");
  printf("\t[-m in the summary, print ms1, ms2, and ms3 in addition to ms0; default: print only ms0]\n");
  printf("\t[-d debugging mode]\n");
  printf("\t[-h print this help message and exit]\n");
  printf("\t[-v print program information and exit]\n");
  exit(0);
}

void printSplash(char *version){
  printf("%s %s\n",progname(),version);
  printf("Written by Bernhard Haubold.\n");
  printf("Distributed under the GNU General Public License.\n");
  printf("Please send bug reports to haubold@evolbio.mpg.de\n");
  exit(0);
}
