/***** prop.c *************************************
 * Description: Read amino acid properties.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Wed Nov  2 10:54:58 2016
 **************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <float.h>
#include "tab.h"
#include "code.h"
#include "eprintf.h"

int checkAa1(char *aa, Code *c){
  int i;

  for(i=0;i<20;i++)
    if(toupper(aa[0]) == c->aa1[i][0])
      return i;
  
  return -1;
}

int checkAa3(char *aa, Code *c){
  int i, ind;
  char *t;

  t = (char *)emalloc(4);
  t[3] = '\0';
  ind = -1;
  for(i=0;i<20;i++){
    t[0] = toupper(aa[0]);
    t[1] = tolower(aa[1]);
    t[2] = tolower(aa[2]);
    if(strcmp(c->aa3[i],t) == 0){
      ind = i;
      break;
    }
  }
  free(t);
  return ind;
}

int checkAa(char *aa, Code *c){
  int l, ind;

  l = strlen(aa);
  if(l == 1)
    ind = checkAa1(aa,c);
  else if(l == 3)
    ind = checkAa3(aa,c);
  else
    ind = -1;

  return ind;  
}

void aaError(char *aa, int lc){
  fprintf(stderr,"ERROR in properties file: %s in line %d is not a valid amino acid designation.\n",aa,lc);
  exit(-1);
}

void duplicationWarning(char *aa, int lc){
  fprintf(stderr,"WARNING: %s is specified repeatedly on line %d of the properties file.\n",aa,lc);
}

void incompletenessError(char *aa){
  fprintf(stderr,"ERROR: %s is not specified in the properties file.\n",aa);
  exit(-1);
}

void printProp(Code *c){
  int i;
  
  for(i=0;i<20;i++)
    printf("%s\t%.2f\n",c->aa3[i],c->pro[i]);
}

void getProp(FILE *fp, Code *c){
  char *aa, *line;
  double p;
  int i, lc, ind;
  
  c->pro = (double *)emalloc(20*sizeof(double));
  for(i=0;i<20;i++)
    c->pro[i] = DBL_MAX;
    
  lc = 0;
  while((line = tabGetLine(fp)) != NULL){
    lc++;
    if(line[0] == '#')
      continue;
    aa = tabField(0);
    p = atof(tabField(1));
    ind = checkAa(aa,c);
    if(ind < 0)
      aaError(aa,lc);
    else{
      if(c->pro[ind] != DBL_MAX)
	duplicationWarning(aa,lc);
      c->pro[ind] = p;
    }
  }
  for(i=0;i<20;i++)
    if(c->pro[i] == DBL_MAX)
      incompletenessError(c->aa3[i]);
}

